export var RenderableType;
(function (RenderableType) {
    RenderableType[RenderableType["Primitive"] = 0] = "Primitive";
    RenderableType[RenderableType["Composite"] = 1] = "Composite";
})(RenderableType || (RenderableType = {}));
export class Primitive {
    constructor() {
        this.retained = false;
        this.kind = RenderableType.Primitive;
    }
    retain() {
        this.retained = true;
        return this;
    }
}
export class Composite {
    constructor() {
        this.kind = RenderableType.Composite;
    }
    scope() {
        return null;
    }
    props(coords) {
        return {};
    }
    retain() {
        function recurse(renderable) {
            if (Array.isArray(renderable)) {
                for (const element of renderable) {
                    recurse(element);
                }
            }
            else if (renderable.kind === RenderableType.Composite) {
                recurse(renderable.children());
            }
            else if (renderable.kind === RenderableType.Primitive) {
                renderable.retained = true;
            }
        }
        recurse(this.children());
        return this;
    }
    dispose() {
        function recurse(renderable) {
            if (Array.isArray(renderable)) {
                for (const element of renderable) {
                    recurse(element);
                }
            }
            else if (renderable.kind === RenderableType.Composite) {
                recurse(renderable.children());
            }
            else if (renderable.kind === RenderableType.Primitive) {
                renderable.retained = false;
                renderable.dispose();
            }
        }
        recurse(this.children());
    }
}
//# sourceMappingURL=common.js.map