import { CoordinateSystem, Kind } from "./coordinate-system";
export function createCartesianCoordinateSystem(xscale, yscale) {
    return new CartesianCoordinateSystem(xscale, yscale);
}
export class CartesianCoordinateSystem extends CoordinateSystem {
    constructor(xscale, yscale) {
        super();
        this.xscale = xscale;
        this.yscale = yscale;
        this.kind = Kind.Cartesian;
        const xglsl = xscale.glsl
            .replace("toDomain", "toXDomain")
            .replace("toRange", "toXRange");
        const yglsl = yscale.glsl
            .replace("toDomain", "toYDomain")
            .replace("toRange", "toYRange");
        this.glsl = `
    uniform vec2 xdomain, ydomain;
    uniform vec2 xrange, yrange;

    ${xglsl}
    ${yglsl}

    vec2 toRange(vec2 v) {
      return vec2(
        toXRange(v.x, xdomain, xrange),
        toYRange(v.y, ydomain, yrange)
      );
    }

    vec2 toDomain(vec2 v) {
      return vec2(
        toXDomain(v.x, xdomain, xrange),
        toYDomain(v.y, ydomain, yrange)
      );
    }
  `;
    }
    toRange(v) {
        return [this.xscale.toRange(v[0]), this.yscale.toRange(v[1])];
    }
    toDomain(v) {
        return [this.xscale.toDomain(v[0]), this.yscale.toDomain(v[1])];
    }
    scope(regl) {
        return regl({
            uniforms: {
                xdomain: regl.prop("xdomain"),
                ydomain: regl.prop("ydomain"),
                xrange: regl.prop("xrange"),
                yrange: regl.prop("yrange"),
            },
        });
    }
    props() {
        return {
            xdomain: this.xscale.domain,
            ydomain: this.yscale.domain,
            xrange: this.xscale.range,
            yrange: this.yscale.range,
        };
    }
}
//# sourceMappingURL=cartesian.js.map