export function createDataset(regl, data, auto = true) {
    if (isDataset(data)) {
        return data;
    }
    return new Dataset(regl, data, auto);
}
// Added to keep the public API the same. I.e., like createRects this function
// expects CandyGraph as the first argument.
export function createDatasetPublic(cg, data, auto = false) {
    return createDataset(cg.regl, data, auto);
}
function isDataset(obj) {
    return obj.constructor === Dataset;
}
export class Dataset {
    constructor(regl, data, auto) {
        this.auto = auto;
        this._length = 0;
        this._disposed = false;
        this.update = (data) => {
            if (this._disposed) {
                throw new Error("This DataSet cannot be updated, it's already been disposed.");
            }
            if (typeof data === "number") {
                this.buffer([data]);
                this._length = 1;
            }
            else if (ArrayBuffer.isView(data)) {
                this.buffer(data);
                this._length = data.length;
            }
            else if (Array.isArray(data)) {
                this.buffer(data);
                if (typeof data[0] === "number") {
                    this._length = data.length;
                }
                else {
                    this._length = data.length * data[0].length;
                }
            }
            return this;
        };
        this.buffer = regl.buffer(1);
        this.update(data);
    }
    dispose() {
        this.buffer.destroy();
        this._disposed = true;
    }
    disposeIfAuto() {
        if (this.auto) {
            this.dispose();
        }
    }
    count(size) {
        this.assertFits(size);
        return this._length / size;
    }
    divisor(instances, size) {
        const count = this.count(size);
        if (count === 1) {
            return instances;
        }
        return 1;
    }
    assertFits(size) {
        if (this._length % size !== 0) {
            throw new Error(`Attempted to use incompatible size ${size} with data of length ${this._length}.`);
        }
    }
}
//# sourceMappingURL=dataset.js.map